// ---------------------------------------------------------------------------------------------------------------
// File        | PZ_InsideBar.mq4
// Description | Detects inside bars, also known as master candles.
// Copyright   | Point Zero Trading Solutions
// Website     | http://www.pointzero-trading.com        
// --------------------------------------------------------------------------------------------------------------- 
#property copyright "Point Zero Trading Solutions"
#property link      "http://www.pointzero-trading.com"

//---- indicator settings
#property indicator_chart_window
#property indicator_buffers 0

//---- constants 
#define  ShortName         "PZ Inside Bar"
#define  OLabel            "PZIB"
#define  Shift             1

//---- indicator parameters

extern bool   Only_Body_Candle      = 0;
extern string St_Ex                 = "------- Indicator Settings";
extern int    MinRange              = 1;
extern int    MaxRange              = 30;
extern int    MaxHistoryBars        = 5000;
extern string Se_Ex3                = "------- Drawing Boxes";
extern color  BoxColor_M1               = clrLimeGreen;
extern color  BoxColor_M5               = clrGreen;
extern color  BoxColor_M15              = clrBlue;
extern color  BoxColor_M30              = clrAquamarine;
extern color  BoxColor_H1               = clrGold;
extern color  BoxColor_H4               = clrMagenta;
extern color  BoxColor_D1               = clrRed;
extern color  BoxColor_W1               = clrAqua;
extern color  BoxColor_MN1              = clrWhite;

extern int    BoxSize               = 2;
extern string Alerts_ex             = "------- Alerts";
extern string AlertCaption          = "My Alert";
extern bool   DisplayAlerts         = false;
extern bool   EmailAlerts           = false;
extern bool   SoundAlerts           = false;
extern string SoundFile             = "alert2.wav";

//---- Alerts
static datetime TimeStamp;
static int AlertCount = 1;
static bool AlertTriggered = false;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   // Delete old Objects 
   DeleteObjects();
   
   // Name and labels
   IndicatorShortName(ShortName);
   
   // Comment
   Comment("");
   
   // Bye
   return(0);
}
//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int DeleteObjects()
{
   int obj_total=ObjectsTotal();
   for(int i = obj_total - 1; i >= 0; i--)
     {
       string label = ObjectName(i);
       if(StringFind(label, OLabel)==-1) continue;
       ObjectDelete(label); 
     }     
   return(0);
}
int deinit()
{
   DeleteObjects();return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars=IndicatorCounted();
   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);

     // More vars here too...
     
     
     // Check the signal foreach bar
     for(int i = limit; i >= 0; i--)
     {       
         // Ignore more than X past bars 
         if(i > MaxHistoryBars) continue;  
         
         // No Alert 
         AlertTriggered = false;
         
         // Some Vars I need
         if(PERIOD_M1>=Period())   GET(PERIOD_M1,BoxColor_M1 ,i ) ;
         if(PERIOD_M5>=Period())   GET(PERIOD_M5,BoxColor_M5 ,i ) ;
         if(PERIOD_M15>=Period())   GET(PERIOD_M15,BoxColor_M15 ,i ) ;
         if(PERIOD_M30>=Period())   GET(PERIOD_M30,BoxColor_M30 ,i ) ;
         if(PERIOD_H1>=Period())   GET(PERIOD_H1,BoxColor_H1 ,i ) ;
         if(PERIOD_H4>=Period())   GET(PERIOD_H4,BoxColor_H4 ,i ) ;
         if(PERIOD_D1>=Period())   GET(PERIOD_D1,BoxColor_D1 ,i ) ;
         if(PERIOD_W1>=Period())   GET(PERIOD_W1,BoxColor_W1 ,i ) ;
         if(PERIOD_MN1>=Period())   GET(PERIOD_MN1,BoxColor_MN1 ,i ) ;
           
     }
   
   
   
   
   
     
   // Alerts
   if(TimeStamp != Time[0])
   {
         if(AlertTriggered && AlertCount == 0)
         {
            if(DisplayAlerts == true) { Alert( ShortName +" ("+ AlertCaption +") ["+ Symbol() +"] Inside Bar"); }
            if(EmailAlerts == true)   { SendMail(ShortName +" ("+ AlertCaption +") ["+ Symbol() +"]", "["+ Symbol() +"] Inside Bar"); }
            if(SoundAlerts == true)   { PlaySound(SoundFile); }
         } 
         TimeStamp = Time[0];
         AlertCount = 0;
   }
   
   // Bye Bye
   return(0);
}
     int gila[PERIOD_MN1] ;
     int rntf[PERIOD_MN1];
     
    int GET(int tf,color as ,int ii ) { 
        if(Time[ii]<gila[tf])return(0);
            
           int GH1=GH(ii,tf);
           int GL1=GL(ii,tf);
             rntf[tf] =  MathMin(GH1,GL1);
           
           if(rntf[tf]>=MinRange&&rntf[tf]<=MaxRange) 
            {
               AlertTriggered = true;
            if(DrawRectangle(tf,"H1",ii-rntf[tf], rntf[tf]+1, as, false)){gila[tf]=Time[ii]+tf*rntf[tf]*60;  }
                 
            } 
         }  


bool DrawRectangle(int bb,string aaa,int shift, int pshift, color vcolor, bool track = true)
{
   // Time
   datetime x1 = iTime(Symbol(),bb,shift);
   datetime x2 = iTime(Symbol(),bb,shift+pshift-1);
   
   // Name
   string label = OLabel +"Rect-"+ x2;
   if(ObjectFind(label) != -1) ObjectDelete(label);
    
   // Distance
   int dist = shift + pshift;
   
   // Highest and lowest
   double rhigh = iHigh(Symbol(),bb,shift+pshift-1);
   double rlow  = iLow(Symbol(),bb,shift+pshift-1 );
   
   
   // Create
   ObjectCreate(label, OBJ_RECTANGLE, 0, x1, rlow, x2, rhigh);
   ObjectSet(label, OBJPROP_COLOR, vcolor);
   ObjectSet(label, OBJPROP_WIDTH, BoxSize);
   ObjectSet(label, OBJPROP_BACK, false);
   return(true);
}

 

 

int GH(int x,int v)
{ 
double a =iHigh(Symbol(),v,x);
 //double b =iLow (Symbol(),Period(),x);

int jum =0;
for(int y=x-1;y>1;y--){
double a1 =iHigh(Symbol(),v,y);
if(Only_Body_Candle)a1=MathMax(iOpen(Symbol(),v,y),iClose(Symbol(),v,y));

//double b1 =iLow (Symbol(),Period(),y);

if(a>a1)jum++;else break;
}  
return(jum);
} 


int GL(int x,int v)
{ 
double a =iLow(Symbol(),v,x);
 
//double b =iLow (Symbol(),Period(),x);

int jum =0;
for(int y=x-1;y>1;y--){
double a1 =iLow(Symbol(),v,y);
if(Only_Body_Candle)a1=MathMin(iOpen(Symbol(),v,y),iClose(Symbol(),v,y));

//double b1 =iLow (Symbol(),Period(),y);
 
if(a<a1)jum++;else break;
}   
return(jum);

}      